/*
 * Bundesliga is Copyright (c) 1997-2000 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "bl.h"
#include "blrcp.h"

struct {
  FieldPtr m1, m2;
} editMatch[10];

static RectangleType eRect;
static PointType ePt;
static CharPtr MItems[20];
static DateTimeType date;
static Short editMatchDay;

/**
 * Edit Form
 * purpose: edit the fixture by drag and drop
 */
static void SetDateMatchdayLabel()
{
  static Char buf[3];
  ControlPtr ctlP = Id2Ptr(ID_FrmEditDatum);
  char *label = CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  DateToAscii(date.month, date.day, date.year, 
  	      sysPrefs.longDateFormat, label);
  //CtlDrawControl(Id2Ptr(ID_FrmEditDatum));
  StrIToA(buf, editMatchDay+1);
  FldSetTextPtr(Id2Ptr(ID_FrmEditFld), buf);
}

void InitFormEdit()
{
  int i;
  for (i=0; i<10; i++) {
    Boolean usable = i<(prefs.NumTeams>>1);
    editMatch[i].m1 = Id2Ptr(ID_FrmEditMann+i);
    editMatch[i].m2 = Id2Ptr(ID_FrmEditMann+i+10);
    FldSetUsable(editMatch[i].m1, usable);
    FldSetUsable(editMatch[i].m2, usable);
  }
  if (prefs.NumTeams % 2 == 1) {
    FldSetUsable(editMatch[prefs.NumTeams>>1].m2, true);
  }
  if (prefs.NumTeams<19) {
    for (i=0; i<9; i++) {
      FrmMoveObject(ID_FrmEditMann+i,0,i);
      FrmMoveObject(ID_FrmEditMann+10+i,0,i);
    }
  }
}

static void DrawFormEdit()
{
  int i, g, m1, m2;
  ULong mask = 0UL;
  for (i=0; i<(prefs.NumTeams>>1); i++) {
    g = programme[editMatchDay*10+i];
    FldSetTextPtr(editMatch[i].m1, M1vSpNr(g));
    FldSetTextPtr(editMatch[i].m2, M2vSpNr(g));
    m1 = g / 100 - 1;
    m2 = g % 100 - 1;
    /* remember which teams play this round */
    mask |= ( 1UL<<m1 ) | ( 1UL<<m2 );
  }
  if (prefs.NumTeams % 2 == 1) {
    for (i=0; i<prefs.NumTeams; i++) {
      /* set team that doesn't play this round */
      if ( (mask & (1UL<<i)) == 0 ) { 
	FldSetTextPtr(editMatch[prefs.NumTeams>>1].m2,team[i]);
	break;
      }
    }
  }

  if (Datum[editMatchDay] == 0) {
    TimSecondsToDateTime(TimGetSeconds(), &date);
  } else {
    if (Datum[editMatchDay]<100000) /* preserve compatibility for v0.4 */
      Datum[editMatchDay] = (Datum[editMatchDay] - 1461) * 86400;
    TimSecondsToDateTime(Datum[editMatchDay], &date);
  }
  SetDateMatchdayLabel();
}

/*
 * swap contents of the two fields
 */
static void exchange(FieldPtr m1, FieldPtr m2)
{
  CharPtr s1, s2;
  s1 = FldGetTextPtr(m1);
  s2 = FldGetTextPtr(m2);
  FldSetTextPtr(m1, s2);
  FldSetTextPtr(m2, s1);
  FldDrawField(m1);
  FldDrawField(m2);
}

static FieldPtr getTeamUnder(FieldPtr f)
{
  FieldAttrType attr;
  FldGetAttributes(f, &attr);
  if (!attr.usable) /* is field used */
    return NULL;
  FldGetBounds(f, &eRect);
  return f;
}

static FieldPtr teamUnder(SWord x, SWord y)
{
  int div = (prefs.NumTeams==20)?11:12;

  if (y<30 || y>137 || (x>75 && x<80) || x>145)
    return NULL;

  y = (y-30) / div;
  if (x<80) { /* left half  */
    return getTeamUnder(editMatch[y].m1);
  }
  /* right half */
  return getTeamUnder(editMatch[y].m2);
}

static void dragTeam(SWord x, SWord y)
{
  WinInvertRectangle(&eRect, 2);
  RctOffsetRectangle(&eRect, x-ePt.x, y-ePt.y);
  WinInvertRectangle(&eRect, 2);
  ePt.x = x;
  ePt.y = y;
}

/*
 * return number of team in team table of argument "s"
 */
static int MNrvString(CharPtr s)
{
  int i;

  /*
   * I've only mangled pointers, so I can compare the pointers
   */
  for (i=0; i<prefs.NumTeams; i++) {
    if (team[i] == s)
      return i+1;
  }
  return 0;
}

/*
 * correct settings in fixture according to drag & drop actions before;
 * if necessary reset results - the user was warned about this!
 */
static void UpdateSpielplan()
{
  int i, m1, m2;
  for (i=0; i<(prefs.NumTeams>>1); i++) {
    m1 = MNrvString(FldGetTextPtr(editMatch[i].m1));
    m2 = MNrvString(FldGetTextPtr(editMatch[i].m2));
    if (m1 && m2) {
      programme[editMatchDay*10+i] = m1*100 + m2;
      results[editMatchDay*10+i] = 0xffff;
    }
  }
}

/*
 * are there any results entered for this match day?
 */
static Boolean FormHasResults()
{
  int i;
  for (i=0; i<(prefs.NumTeams>>1); i++) {
    if (programme[editMatchDay*10+i] && 
	(results[editMatchDay*10+i] != 0xffff) )
      return true;
  }
  return false;
}

Boolean EditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static FieldPtr lastTeam;
  FieldPtr newTeam;
  static Boolean hasChanged = false;
  
CALLBACK_PROLOGUE
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    handled = true;
    editMatchDay = matchDay;
    if (FormHasResults()) {
      if (FrmAlert(ID_FrmHasResults) == 1) {
        /* don't delete results, but return to form immediately */
        FrmReturnToForm(0);
        break;
      }
    }
    DrawFormEdit();
    FrmDrawForm(FrmGetActiveForm());
    break;

  case penDownEvent:
    /* are we over a field with a team? */
    if ((lastTeam = teamUnder(e->screenX, e->screenY))) {
      WinInvertRectangle(&eRect, 2);
      ePt.x = e->screenX;
      ePt.y = e->screenY;
      handled = true;
    }
    break;

  case penMoveEvent:
    /* move rectangle */
    if (lastTeam) {
      dragTeam(e->screenX, e->screenY);
      handled = true;
    }
    break;

  case penUpEvent:
    /* swap teams */
    if (lastTeam) {
      WinInvertRectangle(&eRect, 2);
      if ((newTeam = teamUnder(e->screenX, e->screenY))) {
	exchange(lastTeam,newTeam);
        hasChanged = true;
	handled = true;
      }
    }
    break;

  case ctlSelectEvent:
    /* "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton) {
      if (hasChanged) {
        UpdateSpielplan();
        Datum[editMatchDay] = TimDateTimeToSeconds(&date);
      }
      FrmReturnToForm(0);
      if (hasChanged) 
	DrawBegegnungen(true);
      handled = true; 
      break;
    }

    /* "Cancel" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+1) {
      FrmReturnToForm(0);
      handled = true; 
      break;
    }

    /* "Left" / "Right" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+2 ||
        e->data.ctlSelect.controlID == ID_FrmEditButton+3) {
      handled = true;
      if ( (editMatchDay > 0 && e->data.ctlSelect.controlID == ID_FrmEditButton+2) 
         || (editMatchDay < prefs.Spieltage-1 && e->data.ctlSelect.controlID == ID_FrmEditButton+3) ){
        if (hasChanged) {
          UpdateSpielplan();
          Datum[editMatchDay] = TimDateTimeToSeconds(&date);
          hasChanged = false;
        }
        editMatchDay += (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?-1:1;
        if (FormHasResults()) {
          if (FrmAlert(ID_FrmHasResults) == 1) {
            editMatchDay += (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?1:-1;
            break;
          }
        }
        DrawFormEdit();
        FrmDrawForm(FrmGetActiveForm());
      }
      break;
    }

    /* "Date" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDatum) {
      if ( SelectDay(&date.month, &date.day, &date.year, CHOOSE_DATE) ) {
	SetDateMatchdayLabel();
        FrmDrawForm(FrmGetActiveForm());
        hasChanged = true;
      }
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines handling editing and entering teams list
 */
void InitFormTeams()
{
  SelectedDBInfo info;
  ListPtr p;
  int i;

  SelectedDB(DBNr, &info);
  FrmCopyLabel(FrmGetActiveForm(), ID_FrmTeamsName, info.dbName);

  for (i=0; i<prefs.NumTeams; i++) {
    MItems[i] = team[i];
  }

  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  LstSetListChoices(p, MItems, prefs.NumTeams);
  LstSetHeight(p, 10);
}

static int TeamsListGetSelection()
{
  ListPtr p;
  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  return LstGetSelection(p);
}

static void RedrawList(Word ID_Form, Word ID_List)
{
  ListPtr p;
  p = FrmGetListPtr(ID_Form, ID_List);
  LstEraseList(p);
  LstDrawList(p);
}

Boolean TeamsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Button "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton) {
      FrmReturnToForm(0);
      DrawBegegnungen(true);
      handled = true; 
      break;
    }

    /* Button "Edit" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton+1) {
      FrmPopupForm(ID_FrmEnterMann);
      handled = true; 
      break;
    }

    /* Button "Handicap" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton+2) {
      FrmPopupForm(ID_FrmHandicap);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/*
 * helper function to acquire a handle if necessary and set contents
 * of a field.
 */
void SetField(Word ID, int size, CharPtr val)
{
  FieldPtr fld;
  VoidHand handle;
  CharPtr  name;

  fld = Id2Ptr(ID);
  if (!(handle = (VoidHand)FldGetTextHandle(fld))) {
    handle = MemHandleNew(size*sizeof(char));
    name = MemHandleLock(handle);
    StrCopy(name, val);
    MemHandleUnlock(handle);
    FldSetTextHandle(fld, (Handle)handle);
  } else {
    name = MemHandleLock(handle);
    StrCopy(name, val);
    MemHandleUnlock(handle);
  }
}

/**
 * routines handling the "Enter Team" form
 */
static void DrawFormEnterMann()
{
  int nr = TeamsListGetSelection();

  if (nr == -1)
    return;

  SetField(ID_FrmEnterMannFld,18,team[nr]);
}

static void SetEnteredMann()
{
  int nr = TeamsListGetSelection();
  StrCopy(team[nr], FldGetTextPtr(Id2Ptr(ID_FrmEnterMann)));
}

Boolean EnterMannFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    DrawFormEnterMann();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmEnterMannButton:
      SetEnteredMann();

    /* Cancel */
    case ID_FrmEnterMannButton+1:
      FrmReturnToForm(0);
      RedrawList(ID_FrmTeams, ID_FrmTeamsList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines handling creation of a new database
 */
static void InitFrmNewDB()
{
  int i, s[3] = { 19, 3, 3 };
  for (i=0; i<2; i++) {
    SetField(ID_FrmNewDBFld, s[i], "");
  }
}

/*
 * create database according to entries in form
 */
static void ReadFormNewDBValues()
{
  CharPtr name, nr, rds;
  DmOpenRef db;
  BlDatabasePreferencesType newPrefs;
  int n = 0, m = 0;

  nr = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+1));
  rds = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+2));

  if (nr)  n = StrAToI(nr);
  if (rds) m = StrAToI(rds);

  if (n < 3 || n > 20 || m < n-1 || m > 52) {
    FrmAlert(ID_FrmAlertNumTeams);
    return;
  }

  name = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld));

  newPrefs.NumTeams  = n;
  newPrefs.Spieltage = m;
  newPrefs.matchDay  = 0;
  newPrefs.Version   = 5;
  
  if ( name && (db = CreateDatabase(name)) ) {
    InitNewDatabase(db, &newPrefs);
    DmCloseDatabase(db);
  }
}

Boolean NewDBFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmNewDB();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    /* Ready */
    case ID_FrmNewDBButton:
      ReadFormNewDBValues();
      ChooseDB();

    /* Cancel */
    case ID_FrmNewDBButton+1:
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines handling renaming a database name
 */
static void InitFrmRenameLeague()
{
  Word i = LstGetSelection(FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList));
  SetField(ID_FrmRenameLeagueFld, 26, Items[i]);
}

static void RenameDatabaseFile()
{
  CharPtr name = FldGetTextPtr(Id2Ptr(ID_FrmRenameLeagueFld));
  if (name) {
    RenameDatabase(LstGetSelection(FrmGetListPtr(ID_FrmLeague, 
						 ID_FrmLeagueList)), name);
  }
}

Boolean RenameLeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;

CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmRenameLeague();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmRenameLeagueButton:
      RenameDatabaseFile();

    /* Cancel */
    case ID_FrmRenameLeagueButton+1:
      FrmReturnToForm(0);
      RedrawList(ID_FrmLeague, ID_FrmLeagueList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routine that handles the settings dialog
 */
Boolean SettingsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  ListPtr pLst = Id2Ptr(ID_FrmSettingsPopup);
  ControlPtr pCtl = Id2Ptr(ID_FrmSettingsButton+4);
  static Byte options;
  static Byte leagueOptions;

CALLBACK_PROLOGUE

  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    /* read preferences and set form according to these */
    frm = FrmGetActiveForm();
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 2), 
		       appPrefs.options & editMode);
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 3), 
		       appPrefs.options & dontSaveEM);
    CtlSetLabel(pCtl, LstGetSelectionText(pLst, prefs.options & twoPoints));
    options = appPrefs.options;
    leagueOptions = prefs.options;
    FrmDrawForm(frm);
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmSettingsButton: /* Ready => save settings */
      appPrefs.options = options; 
      prefs.options = leagueOptions;

    case ID_FrmSettingsButton+1: /* Cancel */
      FrmReturnToForm(0);
      handled = true; 
      break;

    case ID_FrmSettingsButton+2: /* Checkbox "Edit Mode" */
      handled = true; 
      options ^= editMode;
      break;

    case ID_FrmSettingsButton+3: /* Checkbox "Save Edit Mode" */
      handled = true; 
      options ^= dontSaveEM;
      break;

      case ID_FrmSettingsButton+4: /* Listbox 2/3 point rule */
      LstSetSelection(pLst, leagueOptions & twoPoints);
      LstPopupList(pLst);
      switch (LstGetSelection(pLst)) {
      case 0:
	leagueOptions &= ~twoPoints;
	handled = true; 
	break;
      case 1:
	leagueOptions |= twoPoints;
	handled = true; 
	break;
      }
      CtlSetLabel(pCtl, LstGetSelectionText(pLst, leagueOptions & twoPoints));
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

    
/**
 * Handicap Form handling routines; 
 * read values from form and store them in handicap[].
 */
static void ReadFormHandicapValues()
{
  CharPtr sp1, sp2, pen1, pen2;
  int sp1i, sp2i, pen1i, pen2i;
  int nr = TeamsListGetSelection();
  FormPtr frm = FrmGetActiveForm();

  sp1i = sp2i = pen1i = pen2i = 0;

  handicap[nr].whenPointsHandicap = 
    handicap[nr].pointsHandicap = 
    handicap[nr].whenGoalsHandicap = 
    handicap[nr].goalsHandicap = 0;

  /* Points Handicap */
  if (FrmGetControlValue(frm, FrmGetObjectIndex(frm,ID_FrmHandicapButton+2))) {
    pen1 = FldGetTextPtr(Id2Ptr(ID_FrmHandicapFld));
    sp1  = FldGetTextPtr(Id2Ptr(ID_FrmHandicapFld+1));
    if (sp1)  sp1i  = StrAToI(sp1);
    if (pen1) pen1i = StrAToI(pen1);
    handicap[nr].whenPointsHandicap = sp1i;
    handicap[nr].pointsHandicap = pen1i;
  }

  /* Goals Handicap */
  if (FrmGetControlValue(frm, FrmGetObjectIndex(frm,ID_FrmHandicapButton+3))) {
    pen2 = FldGetTextPtr(Id2Ptr(ID_FrmHandicapFld+2));
    sp2  = FldGetTextPtr(Id2Ptr(ID_FrmHandicapFld+3));
    if (sp2)  sp2i  = StrAToI(sp2);
    if (pen2) pen2i = StrAToI(pen2);
    handicap[nr].whenGoalsHandicap = sp2i;
    handicap[nr].goalsHandicap = pen2i;
  }
}

static void InitFormHandicap()
{
  int nr = TeamsListGetSelection();
  FormPtr frm = FrmGetActiveForm();
  Char buf[4];

  FrmCopyLabel(frm, ID_FrmHandicapName, team[nr]);
  if ( handicap[nr].whenPointsHandicap ) {
    FrmSetControlValue(frm, FrmGetObjectIndex(frm,ID_FrmHandicapButton+2) ,1);
    StrIToA(buf, handicap[nr].whenPointsHandicap);
    SetField(ID_FrmHandicapFld+1, 3, buf);
    StrIToA(buf, handicap[nr].pointsHandicap);
    SetField(ID_FrmHandicapFld, 4, buf);
  }
  if ( handicap[nr].whenGoalsHandicap ) {
    FrmSetControlValue(frm, FrmGetObjectIndex(frm,ID_FrmHandicapButton+3),1);
    StrIToA(buf, handicap[nr].whenGoalsHandicap);
    SetField(ID_FrmHandicapFld+3, 3, buf);
    StrIToA(buf, handicap[nr].goalsHandicap);
    SetField(ID_FrmHandicapFld+2, 4, buf);
  }
}

Boolean HandicapFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;

CALLBACK_PROLOGUE

  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    InitFormHandicap();
    FrmDrawForm(frm);
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmHandicapButton:
      ReadFormHandicapValues();

    case ID_FrmHandicapButton+1:
      FrmReturnToForm(0);
      handled = true; 
      break;
    }

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines for handling League form;
 * update DBNr if entry in list was moved.
 */
static void updateDbNr(int i, int j)
{
  if (DBNr == i) { 
    DBNr = j;
  } else if (DBNr == j) {
    DBNr = i;
  }
}

Boolean LeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;
  ListPtr p;
  int i;

CALLBACK_PROLOGUE

  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
    LstSetListChoices(p, Items, nrOfItems);
    LstSetHeight(p, nrOfItems>11?11:nrOfItems);
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmLeagueButton: /* Ready */
      handled = true; 
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawBegegnungen(true);
      break;

    case ID_FrmLeagueButton+1: /* move entry up */
      p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
      i = LstGetSelection(p);
      if (i>0) {
	swapDbInfo(i,i-1);
	updateDbNr(i,i-1);
	LstSetListChoices(p, Items, nrOfItems);
	LstDrawList(p);
	LstSetSelection(p,i-1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+2: /* move entry down */
      p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
      i = LstGetSelection(p);
      if (i<nrOfItems-1) {
	swapDbInfo(i,i+1);
	updateDbNr(i,i+1);
	LstSetListChoices(p, Items, nrOfItems);
	LstDrawList(p);
	LstSetSelection(p,i+1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+3: /* Rename database name */
      FrmPopupForm(ID_FrmRenameLeague);
      handled = true;
    }

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}
